use super::*;

pub trait RegularAtomic: AtomicBase + ModeFactory<Mode = RegularFactoryMode> {
    fn ra_deserialize(inlining: impl Inlining) -> AParseResult<Self>;
    fn ra_extend(self, tail: &[u8]) -> AParseResult<Self>;
}

pub trait AtomicProxy {
    type A: AtomicBase + ModeFactory;

    fn pa_deserialize(inlining: impl Inlining) -> AParseResult<Self::A>;
    fn pa_extend(a: Self::A, tail: &[u8]) -> AParseResult<Self::A>;
}

impl<A: RegularAtomic> AtomicProxy for WithMode<A, RegularFactoryMode> {
    type A = A;

    fn pa_deserialize(inlining: impl Inlining) -> AParseResult<Self::A> {
        A::ra_deserialize(inlining)
    }

    fn pa_extend(a: Self::A, tail: &[u8]) -> AParseResult<Self::A> {
        a.ra_extend(tail)
    }
}

impl<A: AtomicBase + FactoryWithMode> Atomic for A
where
    A::WithMode: AtomicProxy<A = Self>,
{
    fn a_deserialize(inlining: impl Inlining) -> AParseResult<Self> {
        <A::WithMode as AtomicProxy>::pa_deserialize(inlining)
    }

    fn a_extend(self, tail: &[u8]) -> AParseResult<Self> {
        <A::WithMode as AtomicProxy>::pa_extend(self, tail)
    }
}
